#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const package_json_1 = require("../package.json");
const openvpn_unattended_install_1 = require("./openvpn-unattended-install");
const pihole_unattended_install_js_1 = require("./pihole-unattended-install.js");
const program = new commander_1.Command();
program.name(package_json_1.name).version(package_json_1.version);
program
    .command(`setup-openvpn`)
    .description('Sets up SSL for OpenVPN Access Server Web UI')
    .option(`-i, --ip [value]`, `The private ip address of the pi hole ec2 instance (for setting up upstream dns)`)
    .requiredOption(`-d, --domain-name [value]`, `The domain name to use to register the SSL certificate`)
    .requiredOption(`-e, --email [value]`, `The email to use to register the SSL certificate`)
    .requiredOption(`-b, --bucket [value]`, `The S3 bucket to reference when saving generated certificate artifacts or retreiving existing certifate artifacts for reuse`)
    .requiredOption(`-r --region [value]`, `The AWS region to use when using the SDK to communicate with AWS`)
    .requiredOption(`-u, --user-name [value]`, `The username of the client user`)
    .requiredOption(`-p, --user-password [value]`, `The password of the client user (special characters need to be escaped before they are passed here)`)
    .action((...args) => {
    new openvpn_unattended_install_1.SetupOpenVpn(args[0]);
});
program
    .command(`setup-pihole`)
    .description(`Sets up unbound as a recursive DNS provider and sets up unbound to use it`)
    .requiredOption(`-r --region [value]`, `The AWS region to use when using the SDK to communicate with AWS`)
    .requiredOption(`-p, --password [value]`, `The password to login to the web interface (special characters need to be escaped before they are passed here)`)
    .action((...args) => {
    new pihole_unattended_install_js_1.SetupPiHole(args[0]);
});
program.parse(process.argv);
//# sourceMappingURL=index.js.map