"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetupPiHole = void 0;
const chalk_1 = __importDefault(require("chalk"));
const package_json_1 = __importDefault(require("../package.json"));
const fs_1 = __importDefault(require("fs"));
const shelljs_1 = __importDefault(require("shelljs"));
class SetupPiHole {
    constructor(options) {
        this.options = options;
        console.log('options', options);
        this.runSetup();
    }
    runSetup() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                console.log(chalk_1.default.bgGreenBright(`Running pihole unattended setup ${package_json_1.default.version}`));
                this.obtainEc2InstanceMetadata();
                this.configureResolvConf();
                this.createPiHoleConfig();
                this.modifyHosts();
                this.installPiHole();
                this.installUnbound();
                this.runTests();
                this.useUnbound();
                this.installTools();
                console.log(chalk_1.default.bgGreenBright(`Setup completed... pihole is now configured with unbound as a recursive dns service!`));
            }
            catch (e) {
                console.log(chalk_1.default.bgRedBright(`Something failed during pihole setup`), e);
                return;
            }
        });
    }
    obtainEc2InstanceMetadata() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Obtaining ec2 instance metadata`));
            this.instanceId = shelljs_1.default
                .exec(`ec2metadata --instance-id`)
                .toString();
            this.localHostName = shelljs_1.default
                .exec(`ec2metadata --local-hostname`)
                .toString();
            this.localIpv4 = shelljs_1.default
                .exec(`ec2metadata --local-ipv4`)
                .toString();
            console.log(chalk_1.default.yellowBright(`instanceId`), chalk_1.default.cyanBright(this.instanceId));
            console.log(chalk_1.default.yellowBright(`localIpv4`), chalk_1.default.cyanBright(this.localIpv4));
            console.log(chalk_1.default.yellowBright(`localHostName`), chalk_1.default.cyanBright(this.localHostName));
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to obtain ec2 instance metadata`), e);
            throw e;
        }
    }
    configureResolvConf() {
        console.log(chalk_1.default.bgGreenBright('Configuring /etc/resolv.conf before installation'));
        try {
            shelljs_1.default.exec(`sudo sed -i -e 's/nameserver 127.0.0.53/nameserver 127.0.0.1/g' /etc/resolv.conf`);
            shelljs_1.default.exec(`sudo sed '$ a DNS=127.0.0.1' /etc/resolv.conf`);
            shelljs_1.default.exec(`sudo sed '$ a DNSStubListener=no' /etc/resolv.conf`);
            shelljs_1.default.exec(`sudo ln -sf /run/systemd/resolve/resolv.conf /etc/resolv.conf`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to configure /etc/resolv.conf`), e);
            throw e;
        }
    }
    createPiHoleConfig() {
        console.log(chalk_1.default.bgGreenBright(`Creating pihole configuration`));
        try {
            shelljs_1.default.mkdir(`-p`, `/etc/pihole`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to create /etc/pihole`), e);
            throw e;
        }
        try {
            const setupVars = `WEBPASSWORD=
PIHOLE_INTERFACE=eth0
IPV4_ADDRESS=${this.localIpv4}/24
IPV6_ADDRESS=
QUERY_LOGGING=true
INSTALL_WEB=true
DNSMASQ_LISTENING=single
PIHOLE_DNS_1=1.1.1.1
PIHOLE_DNS_2=1.0.0.1
PIHOLE_DNS_3=
PIHOLE_DNS_4=
DNS_FQDN_REQUIRED=true
DNS_BOGUS_PRIV=true
DNSSEC=true
TEMPERATUREUNIT=C
WEBUIBOXEDLAYOUT=traditional
WEBTHEME=default-dark
API_EXCLUDE_DOMAINS=
API_EXCLUDE_CLIENTS=
API_QUERY_LOG_SHOW=all
API_PRIVACY_MODE=false
`;
            fs_1.default.writeFileSync('/etc/pihole/setupVars.conf', setupVars, {
                encoding: 'utf8'
            });
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to create /etc/pihole/setupVars.conf`), e);
            throw e;
        }
    }
    modifyHosts() {
        console.log(chalk_1.default.bgGreenBright(`Modifying /etc/hosts`));
        try {
            console.log(chalk_1.default.gray(`Content of /etc/hosts BEFORE running sed operations`));
            console.log(shelljs_1.default.cat(`/etc/hosts`).toString());
            shelljs_1.default.exec(`sudo sed -i "1s/^/${this.localIpv4} ip-${this.localIpv4} \\n/" /etc/hosts`);
            shelljs_1.default.exec(`sudo sed -i /etc/hosts -e "s/^127.0.0.1 localhost$/127.0.0.1 localhost ${this.localHostName}/"`);
            console.log(chalk_1.default.gray(`Content of /etc/hosts AFTER running sed operations`));
            console.log(shelljs_1.default.cat(`/etc/hosts`).toString());
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to modify /etc/hosts`), e);
            throw e;
        }
    }
    installPiHole() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Installing pihole`));
            shelljs_1.default.exec(`curl -L https://install.pi-hole.net | sudo bash /dev/stdin --unattended`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to install pihole`), e);
            throw e;
        }
        try {
            console.log(chalk_1.default.bgGreenBright(`Update pihole admin password`));
            shelljs_1.default.exec(`pihole -a -p "${this.escapeRegExp(this.options.password)}"`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to update pihole admin password`), e);
            throw e;
        }
        try {
            console.log(chalk_1.default.bgGreenBright(`Adding selected adblock lists upfront to pihole`));
            shelljs_1.default.exec(`pihole -a adlist add "https://www.github.developerdan.com/hosts/lists/ads-and-tracking-extended.txt"`);
            shelljs_1.default.exec(`pihole -a adlist add "https://www.github.developerdan.com/hosts/lists/amp-hosts-extended.txt"`);
            shelljs_1.default.exec(`pihole -a adlist add "https://blocklistproject.github.io/Lists/porn.txt"`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to add adblock lists to pihole`), e);
            throw e;
        }
        try {
            console.log(chalk_1.default.bgGreenBright(`Adding useful regexes`));
            shelljs_1.default.exec(`pihole --regex '^ad([sxv]?[0-9]*|system)[_.-]([^.[:space:]]+\.){1,}|[_.-]ad([sxv]?[0-9]*|system)[_.-]' '^(.+[_.-])?adse?rv(er?|ice)?s?[0-9]*[_.-]' '^(.+[_.-])?telemetry[_.-]' '^adim(age|g)s?[0-9]*[_.-]' '^adtrack(er|ing)?[0-9]*[_.-]' '^advert(s|is(ing|ements?))?[0-9]*[_.-]' '^aff(iliat(es?|ion))?[_.-]' '^analytics?[_.-]' '^banners?[_.-]' '^beacons?[0-9]*[_.-]' '^count(ers?)?[0-9]*[_.-]' '^mads\.' '^pixels?[-.]' '^stat(s|istics)?[0-9]*[_.-]' '^https?://([A-Za-z0-9.-]*\.)?clicks\.beap\.bc\.yahoo\.com/' '^https?://([A-Za-z0-9.-]*\.)?secure\.footprint\.net/' '^https?://([A-Za-z0-9.-]*\.)?match\.com/' '^https?://([A-Za-z0-9.-]*\.)?clicks\.beap\.bc\.yahoo(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?sitescout(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?appnexus(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?evidon(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?mediamath(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?scorecardresearch(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?doubleclick(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?flashtalking(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?turn(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?mathtag(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?googlesyndication(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?s\.yimg\.com/cv/ae/us/audience/' '^https?://([A-Za-z0-9.-]*\.)?clicks\.beap/' '^https?://([A-Za-z0-9.-]*\.)?.doubleclick(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?yieldmanager(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?w55c(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?adnxs(\.\w{2}\.\w{2}|\.\w{2,4})/' '^https?://([A-Za-z0-9.-]*\.)?advertising\.com/' '^https?://([A-Za-z0-9.-]*\.)?evidon\.com/' '^https?://([A-Za-z0-9.-]*\.)?scorecardresearch\.com/' '^https?://([A-Za-z0-9.-]*\.)?flashtalking\.com/' '^https?://([A-Za-z0-9.-]*\.)?turn\.com/' '^https?://([A-Za-z0-9.-]*\.)?mathtag\.com/' '^https?://([A-Za-z0-9.-]*\.)?surveylink/' '^https?://([A-Za-z0-9.-]*\.)?info\.yahoo\.com/' '^https?://([A-Za-z0-9.-]*\.)?ads\.yahoo\.com/' '^https?://([A-Za-z0-9.-]*\.)?global\.ard\.yahoo\.com/'`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to add regexes to pihole`), e);
            throw e;
        }
        try {
            console.log(chalk_1.default.bgGreenBright(`Update gravity database`));
            shelljs_1.default.exec(`pihole -g`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to update gravity db`), e);
            throw e;
        }
    }
    installUnbound() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Installing unbound`));
            shelljs_1.default.exec(`sudo apt-get -qq update`);
            shelljs_1.default.exec(`sudo apt-get -qq upgrade -y`);
            shelljs_1.default.exec(`sudo apt-get -qq install unbound -y`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to install unbound package from apt-get`), e);
            throw e;
        }
        try {
            console.log(chalk_1.default.bgGreenBright(`Creating unbound configuration with default values`));
            const unboundConfig = `server:
# If no logfile is specified, syslog is used
# logfile: "/var/log/unbound/unbound.log"
verbosity: 0

interface: 127.0.0.1
port: 5335
do-ip4: yes
do-udp: yes
do-tcp: yes

# May be set to yes if you have IPv6 connectivity
do-ip6: no

# You want to leave this to no unless you have *native* IPv6. With 6to4 and
# Terredo tunnels your web browser should favor IPv4 for the same reasons
prefer-ip6: no

# Use this only when you downloaded the list of primary root servers!
# If you use the default dns-root-data package, unbound will find it automatically
#root-hints: "/var/lib/unbound/root.hints"

# Trust glue only if it is within the server's authority
harden-glue: yes

# Require DNSSEC data for trust-anchored zones, if such data is absent, the zone becomes BOGUS
harden-dnssec-stripped: yes

# Don't use Capitalization randomization as it known to cause DNSSEC issues sometimes
# see https://discourse.pi-hole.net/t/unbound-stubby-or-dnscrypt-proxy/9378 for further details
use-caps-for-id: no

# Reduce EDNS reassembly buffer size.
# Suggested by the unbound man page to reduce fragmentation reassembly problems
edns-buffer-size: 1472

# Perform prefetching of close to expired message cache entries
# This only applies to domains that have been frequently queried
prefetch: yes

# One thread should be sufficient, can be increased on beefy machines. In reality for most users running on small networks or on a single machine, it should be unnecessary to seek performance enhancement by increasing num-threads above 1.
num-threads: 1

# Ensure kernel buffer is large enough to not lose messages in traffic spikes
so-rcvbuf: 1m

# Ensure privacy of local IP ranges
private-address: 192.168.0.0/16
private-address: 169.254.0.0/16
private-address: 172.16.0.0/12
private-address: 10.0.0.0/8
private-address: fd00::/8
private-address: fe80::/10
`;
            fs_1.default.writeFileSync('/etc/unbound/unbound.conf.d/pi-hole.conf', unboundConfig, {
                encoding: 'utf8'
            });
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to update create unbound configuration file`), e);
            throw e;
        }
        try {
            shelljs_1.default.exec(`sleep 30s`);
            shelljs_1.default.exec(`sudo service unbound restart`);
            shelljs_1.default.exec(`sleep 5s`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Error restarting unbound`), e);
            throw e;
        }
    }
    runTests() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Test recursive dns`));
            shelljs_1.default.exec(`dig pi-hole.net @127.0.0.1 -p 5335`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to test recursive dns`), e);
            throw e;
        }
        try {
            console.log(chalk_1.default.green(`Test DNSSec validation`));
            console.log(chalk_1.default.bgGreenBright(`The output below should give a status report of SERVFAIL and no IP address.`));
            shelljs_1.default.exec(`dig sigfail.verteiltesysteme.net @127.0.0.1 -p 5335`);
            console.log(chalk_1.default.bgGreenBright(`The output below should give  NOERROR plus an IP address.`));
            shelljs_1.default.exec(`dig sigok.verteiltesysteme.net @127.0.0.1 -p 5335`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to DNSSec tests`), e);
            throw e;
        }
        try {
            console.log(chalk_1.default.green(`Check if resolv.conf for unbound is disabled`));
            shelljs_1.default.exec(`sudo systemctl status unbound-resolvconf.service`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to determine if unbound resolv.conf is disabled`), e);
            throw e;
        }
    }
    useUnbound() {
        console.log(chalk_1.default.bgGreenBright(`Updating pihole configuration to use unbound dns service`));
        try {
            shelljs_1.default.exec(`sudo sed -i -e 's/PIHOLE_DNS_1=1.1.1.1/PIHOLE_DNS_1=127.0.0.1#5335/g' /etc/pihole/setupVars.conf`);
            shelljs_1.default.exec(`sudo sed -i '/PIHOLE_DNS_2=1.0.0.1/d' /etc/pihole/setupVars.conf`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to update pihole to use unbound`), e);
            throw e;
        }
    }
    installTools() {
        try {
            console.log(chalk_1.default.bgGreenBright(`Installing pihole5-list-tool`));
            shelljs_1.default.exec(`sudo add-apt-repository -y ppa:deadsnakes/ppa`);
            shelljs_1.default.exec(`sudo apt-get -qq update`);
            shelljs_1.default.exec(`sudo apt-get -qq install -y python3.8`);
            shelljs_1.default.exec(`python3 --version`);
            shelljs_1.default.exec(`sudo apt-get -qq install -y python3-venv python3-pip`);
            shelljs_1.default.exec(`python3 -m pip --version`);
            shelljs_1.default.exec(`sudo -H pip3 install pihole5-list-tool`);
        }
        catch (e) {
            console.log(chalk_1.default.bgRedBright(`Failed to determine if unbound resolv.conf is disabled`), e);
            throw e;
        }
    }
    escapeRegExp(input) {
        const source = typeof input === 'string' || input instanceof String ? input : '';
        return source.replace(/[-[/\]{}()*+?.,\\^$|#\s]/g, '\\$&');
    }
    get instanceId() {
        var _a;
        return (_a = this._instanceId) === null || _a === void 0 ? void 0 : _a.replace(/\r?\n|\r/g, '');
    }
    set instanceId(value) {
        this._instanceId = value;
    }
    get localHostName() {
        var _a;
        return (_a = this._localHostName) === null || _a === void 0 ? void 0 : _a.replace(/\r?\n|\r/g, '');
    }
    set localHostName(value) {
        this._localHostName = value;
    }
    get localIpv4() {
        var _a;
        return (_a = this._localIpv4) === null || _a === void 0 ? void 0 : _a.replace(/\r?\n|\r/g, '');
    }
    set localIpv4(value) {
        this._localIpv4 = value;
    }
}
exports.SetupPiHole = SetupPiHole;
//# sourceMappingURL=pihole-unattended-install.js.map